# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class CreateTestSuite(BaseModel):
    update_test_suite: Optional["CreateTestSuiteUpdateTestSuite"] = Field(
        alias="updateTestSuite"
    )


class CreateTestSuiteUpdateTestSuite(BaseModel):
    test_suite: Optional["CreateTestSuiteUpdateTestSuiteTestSuite"] = Field(
        alias="testSuite"
    )


class CreateTestSuiteUpdateTestSuiteTestSuite(BaseModel):
    description: str
    id: str
    org: str
    title: str


CreateTestSuite.model_rebuild()
CreateTestSuiteUpdateTestSuite.model_rebuild()
