# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel


class TestResults(BaseModel):
    test_results: List[Optional["TestResultsTestResults"]] = Field(alias="testResults")


class TestResultsTestResults(BaseModel):
    id: str
    llm_output: str = Field(alias="llmOutput")
    pass_percentage: float = Field(alias="passPercentage")
    pass_percentage_with_optional: float = Field(alias="passPercentageWithOptional")
    result_json: Any = Field(alias="resultJson")
    human_eval: Optional[float] = Field(alias="humanEval")
    human_feedback: str = Field(alias="humanFeedback")
    test: "TestResultsTestResultsTest"
    metadata: Optional[Any]


class TestResultsTestResultsTest(BaseModel):
    test_id: str = Field(alias="testId")
    input_under_test: str = Field(alias="inputUnderTest")


TestResults.model_rebuild()
TestResultsTestResults.model_rebuild()
