# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Any, Optional

from pydantic import Field

from .base_model import BaseModel


class RunSummary(BaseModel):
    run: "RunSummaryRun"


class RunSummaryRun(BaseModel):
    run_id: str = Field(alias="runId")
    pass_percentage: Optional[float] = Field(alias="passPercentage")
    status: str
    run_id: str = Field(alias="runId")
    text_summary: str = Field(alias="textSummary")
    timestamp: Any
    archived: bool
    parameters: Any
    test_suite: "RunSummaryRunTestSuite" = Field(alias="testSuite")


class RunSummaryRunTestSuite(BaseModel):
    title: str


RunSummary.model_rebuild()
RunSummaryRun.model_rebuild()
