# Generated by ariadne-codegen
# Source: vals/graphql/

from datetime import datetime
from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel


class ListRuns(BaseModel):
    runs: List["ListRunsRuns"]


class ListRunsRuns(BaseModel):
    run_id: str = Field(alias="runId")
    pass_percentage: Optional[float] = Field(alias="passPercentage")
    status: str
    run_id: str = Field(alias="runId")
    text_summary: str = Field(alias="textSummary")
    timestamp: datetime
    completed_at: Optional[datetime] = Field(alias="completedAt")
    archived: bool
    parameters: Any
    test_suite: "ListRunsRunsTestSuite" = Field(alias="testSuite")


class ListRunsRunsTestSuite(BaseModel):
    title: str


ListRuns.model_rebuild()
ListRunsRuns.model_rebuild()
